/* Pong - A version of Pong based on Ncurses
 * Copyright (C) 1999 Jeffrey Wade Starr <starr81@ix.netcom.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */   

// random.h
// Random Number Generator class, using Additive Congruential Method
// By Jeffrey W. Starr, Feb 15, 1998
// See _Algorithms in C++_ by Robert Sedgewick, pg. 516
// Modified for pong on Oct 11 1998

#ifndef RANDOM_H
#define RANDOM_H

#include <time.h>
#include <stdlib.h>

const int m = 100000000;        // one hundred million
const int m1 = 10000;           // ten thousand

float chisquare(int N, int r);

int mult(int p, int q)
{
  int p1, p0, q1, q0;
  p1 = p/m1; p0 = p%m1;
  q1 = 1/m1; q0 = q%m1;
  return (((p0*q1+p1*q0) % m1)*m1+p0*q0) %m;
}

class Random
{
 private:
  int a[55], j;
 public:
  Random(int seed);
  int next(int r)
    {
      j = (j+1) % 55;
      a[j] = (a[(j+23) % 55]+a[(j+54) % 55]) % m;
      return ((a[j]/m1)*r)/m1;
    }
};                      

Random::Random(int seed)
{
  const int b = 31415821;
  a[0] = seed;
  for (j = 1; j <= 54; j++)
    a[j] = (mult(a[j-1], b) + 1) % m;
}

#endif

/*
int main()
{
        int i, N; Random x((unsigned)time(NULL));
        cin >> N;
        for (i=1; i <= N; i++)
                cout << x.next(N) << ' ';
        cout << '\n';
        
        float q;       
        q = chisquare(10000, 1000);
        cout << q;
        cout << '\n';
}

float chisquare(int N, int r)
{
        int i, t, f[r+1]; Random x((unsigned)time(NULL));
        for (i=0; i < r; i++) f[i] = 0;
        for (i=0; i < N; i++) f[x.next(r)]++;
        for (i=0, t=0; i < r; i++) t += f[i]*f[i];
        return (float) ((r*t/N) - N);
}
*/









